########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/fa_post.4gl  1.3  Delta: 9/4/90
########################################################################

globals "fa_glob.4gl"

#########################################################################
function fa_post(post_or_check, orig_journal, doc_no, post_no, post_date,
                 doc_date, ref_code, book_ref, doc_desc, depr_date,
                 depr_amount, depr_dept, act_status)
# returns true/false based on pass/fail of the routine
#########################################################################
# This routine takes the FA transaction data as arguments,
# and either posts to FA or checks for an ok posting.
# It is designed to be run in "CHECK" mode during the edit list phase,
# and in "POST" mode during the posting phase.
# It modifies elements in the global record post_fa defined in
# fa_glob.4gl in this directory.
# All routines that post into the 4gen Fixed Assest module should
# do so via this routine.
# This line is called once for each FA transaction
#
# Data elements passed:
#    post_or_check char(5),    "POST" or "CHECK"  tells fa_post to check
#                              the posting (for edit lists), or post it.
#    orig_journal char(2),     original journal (FA, AR, OE, etc...)
#    doc_no integer,           document number (sequential and congruent
#                              for each document within a journal)
#    post_no integer,          posting number (post only)
#    post_date date,           posting date (post only)
#    doc_date date,            accounting date
#    ref_code char(6),         asset code
#    book_ref char(6),         accounting book code
#    doc_desc char(30),        document description
#    depr_date date,           depreciation calculation date
#    depr_amount decimal(12),  depreciation amount
#    depr_dept char(3),        depreciation department
#    act_status char(1)        actvity status (N-normal, V-voided, I-Initial)
#
# status and description variables returned in the post_fa record:
# 0 - Successful
# 1 - fg_error("lib_all","fa_post",1) - Fixed Assets Not Properly Installed
#     - return false
# 3 - fg_error("lib_all","fa_post",3) - Document Number out of Sequence
#     - (warning condition only) - continue
# 4 - fg_error("lib_all","fa_post",4) - Duplicate Document Number Exists
#     - (error condition) - return false
# 5 - fg_error("lib_all","fa_post",5) - Cannot Insert a New Document
#     - (sql insert command failed) - return false
#
# What this process does:
# Check to see if FA is installed.  If not, then all statuses are P(posted)
# If last doc no isn't this doc no (first in a group)
#     if last doc no isnt this doc no - 1, then set warning
#     call trx_post - this (possibly) posts a new document into
#         stxtranr (top level)
#         trx_post returns:
#         0 = if "CHECK" or if "POST" and posted ok
#         1 = insert failed (post only)
#         2 = duplicate found
#     check for duplicate stftranr
#     if "POST", insert into stftranr
# end if (first in group)
# if "CHECK", then return
# (POST) insert the row into stfactvd
#
    define
        post_or_check char(5),
        orig_journal like stxtranr.orig_journal,
        doc_no like stxtranr.doc_no,
        post_no like stxtranr.post_no,
        post_date like stxtranr.post_date,
        doc_date like stxtranr.doc_date,
        ref_code like stxtranr.ref_code,
        book_ref char(6),
        doc_desc like stxtranr.doc_desc,
        depr_date date,
        depr_amount decimal(12),
        depr_dept char(3),
        act_status char(1),
        doc_status smallint

    # initialize document status variable
    let post_fa.status = 0

    # First time... fill post_fa.fa_installed
    if post_fa.fa_installed is null
    then
        whenever error continue
        select tabname into post_fa.description
           from systables where tabname = "stfcntrc"
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            let sqlca.sqlcode = 0
            let post_fa.fa_installed = "N"
            let post_fa.status = 1
            let post_fa.description = "Fixed Assets Not Properly Installed."
            return false
        else
            let post_fa.fa_installed = "Y"
        end if
    end if

    # first of a group
    if doc_no = post_fa.last_doc_no
    then else
        # check for consecutive doc_no
        if post_fa.last_doc_no is not null and
           post_fa.last_doc_no != doc_no - 1
        then
            let post_fa.status = 3
            let post_fa.description = "Document Number out of Sequence"
        end if
        let post_fa.last_doc_no = doc_no

        # post the main document
        call trx_post(post_or_check, orig_journal, doc_no, post_no,
            post_date, doc_date, ref_code, doc_desc) returning doc_status
        case doc_status
            when 1
                let post_fa.status = 5
                let post_fa.description = "Cannot Insert a New Document"
                return false
            when 2
                let post_fa.status = 4
                let post_fa.description = "Duplicate Document Number Exists"
                return false
        end case

        # post the FA document
        if post_or_check = "POST"
        then
            # check for an existing document number
            whenever error continue
            select stftranr.orig_journal from stftranr
            where stftranr.orig_journal = orig_journal and
                stftranr.doc_no = doc_no
            whenever error call error_handler
            if sqlca.sqlcode != notfound
            then
                let post_fa.status = 4
                let post_fa.description = "Duplicate Document Number"
                return false
            end if

            # insert the stftranr record
            whenever error continue
            insert into stftranr values (orig_journal, doc_no)
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_fa.status = 5
                let post_fa.description = "Cannot Insert a New Document"
                return false
            end if
        end if   # POST
    end if   # first document in batch

    # done if CHECK only
    if post_or_check = "CHECK"
    then
        return true
    end if

    # POST the activity - insert the stfactvd row
    whenever error continue
    insert into stfactvd values (orig_journal, doc_no, book_ref, depr_date,
        depr_amount, depr_dept, act_status)
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        let sqlca.sqlcode = 0
        let post_fa.status = 5
        let post_fa.description = "Cannot Insert a New Document"
        return false
    end if

    return true
end function
# fa_post()
