########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../lib.4gm/all.4gs/gl_post.4gl  1.12  Delta: 9/4/90
########################################################################

globals "gl_glob.4gl"

define
    l_prepared smallint

########################################################################
function gl_post(post_or_check, orig_journal, doc_no,
                 post_no, post_date, doc_date, ref_code, doc_desc,
                 inv_chk_no, acct_no, department, amount, debit_credit)
# returns true/false based on pass/fail of the routine
########################################################################
# This routine takes the G/L transaction data as arguments,
# and either posts to G/L or checks for an ok posting.
# It is designed to be run in "CHECK" mode during the edit list phase,
# and in "POST" mode during the posting phase.
# It modifies elements in the global record post_gl defined in
# gl_glob.4gl in this directory.
# All routines that post into the 4gen general ledger module should
# do so via this routine.
# This line is called once for each G/L transaction
#
# Data elements passed:
#    post_or_check char(5),  "POST" or "CHECK"  tells gl_post to check
#                            the posting (for edit lists), or post it.
#    orig_journal char(2),   original journal (AR, OE, etc...)
#    doc_no integer,         document number (sequential and congruent
#                            for each document within a journal)
#    post_no integer,        posting number (post only)
#    post_date date,         posting date (post only)
#    doc_date date,          document entry date
#    ref_code char(6),       customer code
#    doc_desc char(30),      document description
#    inv_chk_no char(12),    invoice or check number
#    acct_no integer,        account number
#    department char(3),     sub-account (department) code
#    amount like stgactvd.amount,   always a positive amount
#    debit_credit char(1)    debit or credit the account by amount
#
# status and description variables returned in the post_gl record:
# 0 - Successful
# 1 - fg_error("lib_all","gl_post",1) - Cannot Read G/L Control Table.
#     - return false
# 3 - fg_error("lib_all","gl_post",3) - Document Number out of Sequence
#     - (warning condition only) - continue
# 4 - fg_error("lib_all","gl_post",4) - Duplicate Document Number Exists
#     - (error condition) - return false
# 5 - fg_error("lib_all","gl_post",5) - Cannot Insert a New Document
#     - (sql insert command failed) - return false
# 6 - fg_error("lib_all","gl_post",6) - Account Not Listed in Chart of Accounts
#     -  return false
#
# What this process does:
# Check to see if G/L is installed.  If not, then all statuses are P(posted)
# If last doc no isn't this doc no (first in a group)
#     if last doc no isnt this doc no - 1, then set warning #4
#     call trx_post - this (possibly) posts a new document into
#         stxtranr (top level)
#         trx_post returns:
#         0 = if "CHECK" or if "POST" and posted ok
#         1 = insert failed (post only)
#         2 = duplicate found
#     if "CHECK", check for duplicate stgtranr
#     if "POST", insert into stgtranr
# if (first in group)
# if "CHECK", then return
# if "POST" then:
#     insert the row into stgactvd
#
# This process is designed to be run in conjunction with the gl_last()
# library function that checks to make sure all credits match all debits
# and returns false if they don't.
#
    define
        post_or_check char(5),
        orig_journal like stxtranr.orig_journal,
        doc_no like stxtranr.doc_no,
        post_no like stxtranr.post_no,
        post_date like stxtranr.post_date,
        doc_date like stxtranr.doc_date,
        ref_code like stxtranr.ref_code,
        doc_desc like stxtranr.doc_desc,
        inv_chk_no like stxckrgd.inv_chk_no,
        acct_no like stgactvd.acct_no,
        department like stgactvd.department,
        amount like stgactvd.amount,
        debit_credit like stgactvd.debit_credit,
        acct_period like stgtranr.acct_period,
        acct_year like stgtranr.acct_year,
        doc_status like stgtranr.status,
        getid integer,      # something to get a rowid into
        wait_win smallint,  # boolean: has wait window been opened?,
        wait_counter smallint, # number of times to wait
        gl_setup_date date,
        trx_status smallint,
        gl_balanced char(1)

    let post_gl.status = 0

    # First time calling this function...
    if l_prepared = 123
    then else
        # check for g/l installed...
        if table_exist("stgcntrc")
          then let post_gl.gl_installed = "Y"
          else let post_gl.gl_installed = "N"
        end if
        # prepare a few SQL statements...
        prepare gtranr_insert from
          "insert into stgtranr values(?,?,?,?,?)"
        prepare gactvd_insert from
          "insert into stgactvd values(?,?,?,?,?,?)"
        prepare xckrgd_insert from
          "insert into stxckrgd values(?,?,?,?,?,?,?,?)"
        prepare curs_1 from
          "select count(*) from stgtranr where orig_journal = ? and doc_no = ?"
        declare count_trx cursor for curs_1
        prepare curs_2 from
          "select rowid from stxckrgr where acct_no = ? and department = ? for update"
        declare lock_ckrg cursor for curs_2
        prepare xckrgr_update from
          "update stxckrgr set acct_no = acct_no where current of lock_ckrg"
        prepare curs_3 from "select gl_balanced, setup_date from stgcntrc"
        declare get_setup_date cursor for curs_3
        let l_prepared = 123
    end if

    if post_gl.gl_installed = "N"
    then
        # default to P - Posted if G/L isn't installed
        let doc_status = "P"
    else
        open get_setup_date
        fetch get_setup_date into gl_balanced, gl_setup_date
        if gl_balanced = "N" and
          (gl_setup_date is null or gl_setup_date > doc_date)
        then
          # set to P - Posted if G/L setup is not complete and
          #          - the setup complete date is later than the document date.
          let doc_status = "P"
        else
          # set to N - Not Posted if G/L setup is completed or
          #          - G/L setup is not complete and setup complete date
          #          - is before the document date
          let doc_status = "N"
        end if
    end if

    if doc_no = post_gl.last_doc_no
    then else
        # first of a group
        if post_gl.last_doc_no is not null and
           post_gl.last_doc_no != doc_no - 1
        then
            let post_gl.status = 3
            let post_gl.description = "Document Number out of Sequence"
        end if
        let post_gl.last_doc_no = doc_no
        let post_gl.db_accum = 0
        let post_gl.cr_accum = 0

        # set the period...
        call what_period(doc_date) returning post_gl.acct_period,
            post_gl.acct_year

        call trx_post(post_or_check, orig_journal, doc_no,
                 post_no, post_date, doc_date, ref_code, doc_desc)
            returning trx_status
        if trx_status = 1
        then
            let post_gl.status = 5
            let post_gl.description = "Cannot Insert A New Document.",
                                      " (stxtranr)"
            return false
        end if
        if trx_status = 2
        then
            let post_gl.status = 4
            let post_gl.description = "Duplicate Document Number Exist.",
                                      " (stxtranr)"
            return false
        end if
        if post_or_check = "POST"
        then
            whenever error continue
            open count_trx using orig_journal, doc_no
            fetch count_trx into post_no
            close count_trx
            whenever error call error_handler
            if post_no > 0
            then
                let post_gl.status = 4
                let post_gl.description = "Duplicate Document Number Exist.",
                                          " (stgtranr)"
                return false
            end if

            # insert the stgtranr record
            whenever error continue
            execute gtranr_insert using
                 orig_journal, doc_no, post_gl.acct_period,
                 post_gl.acct_year, doc_status
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                let post_gl.status = 5
                let post_gl.description = "Cannot Insert A New Document.",
                                          " (stgtranr)"
                return false
            end if
        end if   # POST
    end if   # first document in batch

    # get the chart description & set some accumulators...
    call al_chrtn(acct_no)
        returning post_gl.acct_desc, post_gl.incr_with_crdt

    # al_chrtn returns a "C" or a "D" which is translated back to "Y/N"
    if post_gl.incr_with_crdt = "C"
      then let post_gl.incr_with_crdt = "Y"
      else let post_gl.incr_with_crdt = "N"
    end if

    let post_gl.signed_amount = amount
    if debit_credit = "D"
    then
        if post_gl.incr_with_crdt = "Y"
        then
            let post_gl.signed_amount = amount * "-1"
        end if
        let post_gl.db_accum = post_gl.db_accum + amount
    else
        if post_gl.incr_with_crdt = "N"
        then
            let post_gl.signed_amount = amount * "-1"
        end if
        let post_gl.cr_accum = post_gl.cr_accum + amount
    end if

    if post_gl.acct_desc = "NOT FOUND"
    then
        let post_gl.acct_desc = "ACCOUNT NOT FOUND"
        let post_gl.description = "ACCOUNT NOT FOUND"
        let post_gl.status = 6
        return false
    end if

    if post_or_check = "CHECK"
    then
        let post_gl.status = 0
        return true
    end if

    # POST type...
    # insert the stgactvd row
    whenever error continue
    execute gactvd_insert using
        orig_journal, doc_no, acct_no, department, amount, debit_credit
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        let sqlca.sqlcode = 0
        let post_gl.status = 5
        let post_gl.description = "Cannot Insert A New Document.",
                                  " (stgactvd)"
        return false
    end if

    # insert the stxckrgd row if there is a row in stxckrgr
    # for this account number.  Also, try to obtain a lock
    # on the row so nobody can update while the insert is occuring.
    # try to get a lock...
    let wait_win = false
    let wait_counter = 1
    while true
        whenever error continue
        open lock_ckrg using acct_no, department
        fetch lock_ckrg into getid
        whenever error call error_handler
        case
          when sqlca.sqlcode = notfound
            # this account/department isn't defined in stxckrgd
            exit while
          when sqlca.sqlcode = 0
            # lock has been gotten.  insert the stxckrgd row
            whenever error continue
            execute xckrgr_update
            execute xckrgd_insert using
                orig_journal, doc_no, acct_no, department, inv_chk_no,
                amount, debit_credit, "N"
            whenever error call error_handler
            if sqlca.sqlcode != 0
            then
                let sqlca.sqlcode = 0
                close lock_ckrg
                let post_gl.status = 5
                let post_gl.description = "Cannot Insert A New Document.",
                                          " (stxckrgd)"
                return false
            end if
            close lock_ckrg
            exit while
          otherwise
            # cannot obtain lock.  sleep then try again.
            let wait_counter = wait_counter + 1
            if wait_counter <= 30
            then
              if wait_win = false
              then
                open window gl_postwin at 4,4 with 1 rows, 70 columns
                attribute(white,border)
                display
                  " Waiting for check register table to become available..."
                  at 1,1
                let wait_win = true
              end if
              sleep 1
            else
              let post_gl.status = 5
              let post_gl.description = "Cannot Insert A New Document.",
                                        " (stxckrgd)"
              if wait_win then close window gl_postwin end if
              return false
            end if
        end case
    end while
    # close the wait window if necessary
    if wait_win then close window gl_postwin end if

    return true
end function
# gl_post()

